import { writeFile } from 'fs/promises';
import semver from 'semver';
import createDebugMessages from 'debug';
/**
 * Syncronizes dependencies between {@link SyncDependenciesParam.deepJsonFilePath} and {@link SyncDependenciesParam.packageJsonFilePath}
 */
export async function syncDependencies(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependencies');
    debug({ param });
    const { deepJsonFilePath, packageJsonFilePath: packageJsonPath, } = param;
    const { default: deepJson } = await import(deepJsonFilePath, { assert: { type: 'json' } });
    debug({ deepJson });
    const { default: packageJson } = await import(packageJsonPath, { assert: { type: 'json' } });
    debug({ packageJson });
    if (!packageJson.dependencies) {
        packageJson.dependencies = {};
    }
    const packageJsonMissingDependenciesFromDeepJson = await getPackageJsonMissingDependenciesFromDeepJson({
        deepJson,
        packageJson
    });
    debug({ packageJsonMissingDependenciesFromDeepJson });
    packageJsonMissingDependenciesFromDeepJson.forEach((dependency) => {
        packageJson.dependencies = { ...packageJson.dependencies, [dependency.name]: `~${dependency.version}` };
    });
    const syncDependenciesBasedOnDeepJsonResult = await syncDependenciesBasedOnDeepJson({
        deepJson,
        packageJson
    });
    const syncDependenciesBasedOnPackageJsonResult = await syncDependenciesBasedOnPackageJson({
        deepJson,
        packageJson
    });
    deepJson.dependencies = { ...syncDependenciesBasedOnDeepJsonResult.deepJsonDependencies, ...syncDependenciesBasedOnPackageJsonResult.deepJsonDependencies };
    debug({ deepJsonDependencies: deepJson.dependencies });
    packageJson.dependencies = { ...syncDependenciesBasedOnDeepJsonResult.packageJsonDependencies, ...syncDependenciesBasedOnPackageJsonResult.packageJsonDependencies };
    debug({ packageJsonDependencies: packageJson.dependencies });
    writeFile(deepJsonFilePath, JSON.stringify(deepJson, null, 2));
    writeFile(packageJsonPath, JSON.stringify(packageJson, null, 2));
}
async function syncDependenciesBasedOnDeepJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependenciesBasedOnDeepJson');
    debug({ param });
    const { deepJson, packageJson } = param;
    let packageJsonDependencies = packageJson.dependencies;
    debug({ packageJsonDependencies });
    let deepJsonDependencies = deepJson.dependencies;
    debug({ deepJsonDependencies });
    deepJson.dependencies.forEach((dependency) => {
        const deepJsonDependencyVersionWithoutRange = semver.minVersion(dependency.version)?.version;
        debug({ dependencyVersionWithoutRange: deepJsonDependencyVersionWithoutRange });
        if (!deepJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const packageJsonDependencyVersionWithoutRange = semver.minVersion(packageJson.dependencies[dependency.name])?.version;
        debug({ packageJsonDependencyVersionWithoutRange });
        if (!packageJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const isDeepJsonVersionGreater = semver.gt(deepJsonDependencyVersionWithoutRange, packageJsonDependencyVersionWithoutRange);
        if (isDeepJsonVersionGreater) {
            packageJsonDependencies[dependency.name] = `~${deepJsonDependencyVersionWithoutRange}`;
        }
        else {
            deepJsonDependencies = [...deepJson.dependencies, {
                    name: dependency.name,
                    version: deepJsonDependencyVersionWithoutRange
                }];
        }
    });
    const result = { packageJsonDependencies, deepJsonDependencies };
    debug({ result });
    return result;
}
async function syncDependenciesBasedOnPackageJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:syncDependenciesBasedOnDeepJson');
    debug({ param });
    const { deepJson, packageJson } = param;
    let packageJsonDependencies = packageJson.dependencies;
    debug({ packageJsonDependencies });
    let deepJsonDependencies = deepJson.dependencies;
    debug({ deepJsonDependencies });
    Object.entries(packageJsonDependencies).forEach(([dependencyName, dependencyVersion]) => {
        const packageJsonDependencyVersionWithoutRange = semver.minVersion(dependencyVersion)?.version;
        debug({ dependencyVersionWithoutRange: packageJsonDependencyVersionWithoutRange });
        if (!packageJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const deepJsonDependency = deepJsonDependencies.find(dependency => dependency.name === dependency.name);
        if (!deepJsonDependency)
            return;
        const deepJsonDependencyVersionWithoutRange = semver.minVersion(deepJsonDependency.version)?.version;
        debug({ deepJsonDependencyVersionWithoutRange });
        if (!deepJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const isPackageJsonVersionGreater = semver.gt(packageJsonDependencyVersionWithoutRange, deepJsonDependencyVersionWithoutRange);
        if (isPackageJsonVersionGreater) {
            deepJsonDependencies = [...deepJsonDependencies, {
                    name: dependencyName,
                    version: packageJsonDependencyVersionWithoutRange
                }];
        }
        else {
            packageJsonDependencies[dependencyName] = `~${deepJsonDependencyVersionWithoutRange}`;
        }
    });
    const result = { packageJsonDependencies, deepJsonDependencies };
    debug({ result });
    return result;
}
async function getPackageJsonMissingDependenciesFromDeepJson(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:getPackageJsonMissingDependenciesFromDeepJson');
    const { deepJson, packageJson } = param;
    const missingDependenciesFromDeepJson = deepJson.dependencies.filter((dependency) => packageJson.dependencies[dependency.name] === undefined);
    debug({ missingDependenciesFromDeepJson });
    return missingDependenciesFromDeepJson;
}
